@@ -128,7 +128,7 @@ class Cache {
 //
 abstract class QMatrix {
 	abstract float[] get_Q(int column, int len);
-	abstract float[] get_QD();
+	abstract double[] get_QD();
 	abstract void swap_index(int i, int j);
 };
 
@@ -143,7 +143,7 @@ abstract class Kernel extends QMatrix {
 	private final double coef0;
 
 	abstract float[] get_Q(int column, int len);
-	abstract float[] get_QD();
+	abstract double[] get_QD();
 
 	void swap_index(int i, int j)
 	{
@@ -309,7 +309,7 @@ class Solver {
 	byte[] alpha_status;	// LOWER_BOUND, UPPER_BOUND, FREE
 	double[] alpha;
 	QMatrix Q;
-	float[] QD;
+	double[] QD;
 	double eps;
 	double Cp,Cn;
 	double[] p;
@@ -501,7 +501,7 @@ class Solver {
 
 			if(y[i]!=y[j])
 			{
-				double quad_coef = Q_i[i]+Q_j[j]+2*Q_i[j];
+				double quad_coef = QD[i]+QD[j]+2*Q_i[j];
 				if (quad_coef <= 0)
 					quad_coef = 1e-12;
 				double delta = (-G[i]-G[j])/quad_coef;
@@ -544,7 +544,7 @@ class Solver {
 			}
 			else
 			{
-				double quad_coef = Q_i[i]+Q_j[j]-2*Q_i[j];
+				double quad_coef = QD[i]+QD[j]-2*Q_i[j];
 				if (quad_coef <= 0)
 					quad_coef = 1e-12;
 				double delta = (G[i]-G[j])/quad_coef;
@@ -707,7 +707,7 @@ class Solver {
 					if (grad_diff > 0)
 					{
 						double obj_diff; 
-						double quad_coef=Q_i[i]+QD[j]-2.0*y[i]*Q_i[j];
+						double quad_coef = QD[i]+QD[j]-2.0*y[i]*Q_i[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
@@ -731,7 +731,7 @@ class Solver {
 					if (grad_diff > 0)
 					{
 						double obj_diff; 
-						double quad_coef=Q_i[i]+QD[j]+2.0*y[i]*Q_i[j];
+						double quad_coef = QD[i]+QD[j]+2.0*y[i]*Q_i[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
@@ -953,7 +953,7 @@ final class Solver_NU extends Solver
 					if (grad_diff > 0)
 					{
 						double obj_diff; 
-						double quad_coef = Q_ip[ip]+QD[j]-2*Q_ip[j];
+						double quad_coef = QD[ip]+QD[j]-2*Q_ip[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
@@ -977,7 +977,7 @@ final class Solver_NU extends Solver
 					if (grad_diff > 0)
 					{
 						double obj_diff; 
-						double quad_coef = Q_in[in]+QD[j]-2*Q_in[j];
+						double quad_coef = QD[in]+QD[j]-2*Q_in[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
@@ -1135,16 +1135,16 @@ class SVC_Q extends Kernel
 {
 	private final byte[] y;
 	private final Cache cache;
-	private final float[] QD;
+	private final double[] QD;
 
 	SVC_Q(svm_problem prob, svm_parameter param, byte[] y_)
 	{
 		super(prob.l, prob.x, param);
 		y = (byte[])y_.clone();
 		cache = new Cache(prob.l,(long)(param.cache_size*(1<<20)));
-		QD = new float[prob.l];
+		QD = new double[prob.l];
 		for(int i=0;i<prob.l;i++)
-			QD[i]= (float)kernel_function(i,i);
+			QD[i] = kernel_function(i,i);
 	}
 
 	float[] get_Q(int i, int len)
@@ -1159,7 +1159,7 @@ class SVC_Q extends Kernel
 		return data[0];
 	}
 
-	float[] get_QD()
+	double[] get_QD()
 	{
 		return QD;
 	}
@@ -1169,22 +1169,22 @@ class SVC_Q extends Kernel
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
 		do {byte _=y[i]; y[i]=y[j]; y[j]=_;} while(false);
-		do {float _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 }
 
 class ONE_CLASS_Q extends Kernel
 {
 	private final Cache cache;
-	private final float[] QD;
+	private final double[] QD;
 
 	ONE_CLASS_Q(svm_problem prob, svm_parameter param)
 	{
 		super(prob.l, prob.x, param);
 		cache = new Cache(prob.l,(long)(param.cache_size*(1<<20)));
-		QD = new float[prob.l];
+		QD = new double[prob.l];
 		for(int i=0;i<prob.l;i++)
-			QD[i]= (float)kernel_function(i,i);
+			QD[i] = kernel_function(i,i);
 	}
 
 	float[] get_Q(int i, int len)
@@ -1199,7 +1199,7 @@ class ONE_CLASS_Q extends Kernel
 		return data[0];
 	}
 
-	float[] get_QD()
+	double[] get_QD()
 	{
 		return QD;
 	}
@@ -1208,7 +1208,7 @@ class ONE_CLASS_Q extends Kernel
 	{
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
-		do {float _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 }
 
@@ -1220,14 +1220,14 @@ class SVR_Q extends Kernel
 	private final int[] index;
 	private int next_buffer;
 	private float[][] buffer;
-	private final float[] QD;
+	private final double[] QD;
 
 	SVR_Q(svm_problem prob, svm_parameter param)
 	{
 		super(prob.l, prob.x, param);
 		l = prob.l;
 		cache = new Cache(l,(long)(param.cache_size*(1<<20)));
-		QD = new float[2*l];
+		QD = new double[2*l];
 		sign = new byte[2*l];
 		index = new int[2*l];
 		for(int k=0;k<l;k++)
@@ -1236,7 +1236,7 @@ class SVR_Q extends Kernel
 			sign[k+l] = -1;
 			index[k] = k;
 			index[k+l] = k;
-			QD[k] = (float)kernel_function(k,k);
+			QD[k] = kernel_function(k,k);
 			QD[k+l] = QD[k];
 		}
 		buffer = new float[2][2*l];
@@ -1247,7 +1247,7 @@ class SVR_Q extends Kernel
 	{
 		do {byte _=sign[i]; sign[i]=sign[j]; sign[j]=_;} while(false);
 		do {int _=index[i]; index[i]=index[j]; index[j]=_;} while(false);
-		do {float _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 
 	float[] get_Q(int i, int len)
@@ -1269,7 +1269,7 @@ class SVR_Q extends Kernel
 		return buf;
 	}
 
-	float[] get_QD()
+	double[] get_QD()
 	{
 		return QD;
 	}
@@ -1279,7 +1279,8 @@ public class svm {
 	//
 	// construct and solve various formulations
 	//
-	public static final int LIBSVM_VERSION=291; 
+	public static final int LIBSVM_VERSION=310; 
+	public static final Random rand = new Random();
 
 	private static svm_print_interface svm_print_stdout = new svm_print_interface()
 	{
@@ -1311,7 +1312,7 @@ public class svm {
 		{
 			alpha[i] = 0;
 			minus_ones[i] = -1;
-			if(prob.y[i] > 0) y[i] = +1; else y[i]=-1;
+			if(prob.y[i] > 0) y[i] = +1; else y[i] = -1;
 		}
 
 		Solver s = new Solver();
@@ -1669,7 +1670,7 @@ public class svm {
 		int t,j;
 		int iter = 0, max_iter=Math.max(100,k);
 		double[][] Q=new double[k][k];
-		double[] Qp= new double[k];
+		double[] Qp=new double[k];
 		double pQp, eps=0.005/k;
 	
 		for (t=0;t<k;t++)
@@ -1735,7 +1736,7 @@ public class svm {
 		for(i=0;i<prob.l;i++) perm[i]=i;
 		for(i=0;i<prob.l;i++)
 		{
-			int j = i+(int)(Math.random()*(prob.l-i));
+			int j = i+rand.nextInt(prob.l-i);
 			do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
 		}
 		for(i=0;i<nr_fold;i++)
@@ -2157,7 +2158,7 @@ public class svm {
 			for (c=0; c<nr_class; c++)
 				for(i=0;i<count[c];i++)
 				{
-					int j = i+(int)(Math.random()*(count[c]-i));
+					int j = i+rand.nextInt(count[c]-i);
 					do {int _=index[start[c]+j]; index[start[c]+j]=index[start[c]+i]; index[start[c]+i]=_;} while(false);
 				}
 			for(i=0;i<nr_fold;i++)
@@ -2189,7 +2190,7 @@ public class svm {
 			for(i=0;i<l;i++) perm[i]=i;
 			for(i=0;i<l;i++)
 			{
-				int j = i+(int)(Math.random()*(l-i));
+				int j = i+rand.nextInt(l-i);
 				do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
 			}
 			for(i=0;i<=nr_fold;i++)
