@@ -103,8 +103,12 @@ int main(int argc, char **argv)
 	else
 	{
 		model = svm_train(&prob,&param);
-		svm_save_model(model_file_name,model);
-		svm_destroy_model(model);
+		if(svm_save_model(model_file_name,model))
+		{
+			fprintf(stderr, "can't save model to file %s\n", model_file_name);
+			exit(1);
+		}
+		svm_free_and_destroy_model(&model);
 	}
 	svm_destroy_param(&param);
 	free(prob.y);
@@ -317,9 +321,12 @@ void read_problem(const char *filename)
 		inst_max_index = -1; // strtol gives 0 if wrong format, and precomputed kernel has <index> start from 0
 		readline(fp);
 		prob.x[i] = &x_space[j];
-		label = strtok(line," \t");
+		label = strtok(line," \t\n");
+		if(label == NULL) // empty line
+			exit_input_error(i+1);
+
 		prob.y[i] = strtod(label,&endptr);
-		if(endptr == label)
+		if(endptr == label || *endptr != '\0')
 			exit_input_error(i+1);
 
 		while(1)
